﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.ComponentModel;

namespace DynamicColumnTest
{

    public class KeyPropertyDescriptor : PropertyDescriptor
    {
        string key;
        public KeyPropertyDescriptor(string key):base("Key", null)
        {
            this.key = key;
        }

        #region PropertyDescroptor methods
        public override bool CanResetValue(object component)
        {
            return false;
        }

        public override Type ComponentType
        {
            get { return typeof(DataForBinding); }
        }

        public override object GetValue(object component)
        {
            return key;
        }

        public override bool IsReadOnly
        {
            get { return true; }
        }

        public override Type PropertyType
        {
            get { return typeof(string); }
        }

        public override void ResetValue(object component)
        {
            throw new InvalidOperationException("이 프로퍼티는 ReadOnly로만 동작되어야 한다.");
        }

        public override void SetValue(object component, object value)
        {
            throw new InvalidOperationException("이 프로퍼티는 ReadOnly로만 동작되어야 한다.");
        }

        public override bool ShouldSerializeValue(object component)
        {
            return false;
        }
        #endregion
    }

public class DatePropertyDescriptor : PropertyDescriptor
{
    private IEnumerable<Data> datas;
    private DateTime dateTime;

    public DatePropertyDescriptor(IEnumerable<Data> datas, DateTime dateTime) : 
        base(string.Format("P{0}_{1}_{2}",dateTime.Year, dateTime.Month, dateTime.Day), null)
    {
        // TODO: Complete member initialization
        this.datas = datas;
        this.dateTime = dateTime;
    }

    #region PropertyDescroptor methods
    public override bool CanResetValue(object component)
    {
        return false;
    }

    public override Type ComponentType
    {
        get { return typeof(DataForBinding); }
    }

    public override object GetValue(object component)
    {
        StringBuilder stringBuilder = new StringBuilder();
        foreach (Data data in datas.Where(d => d.DateTime.Day == dateTime.Day && d.DateTime.Month == dateTime.Month))
        {
            stringBuilder.AppendLine(string.Format("{0:00}:{1:00}", (int)data.DateTime.TimeOfDay.TotalHours, data.DateTime.TimeOfDay.Minutes)
                                        + " - " + data.Detail);
        }
        //19 : 56 – 활동 1
        //21 : 56 – 활동 2
        //23: 56 – 활동 3 과 같은 string을 반환한다.
        return stringBuilder.ToString();
    }

    public override bool IsReadOnly
    {
        get { return true; }
    }

    public override Type PropertyType
    {
        get { return typeof(string); }
    }

    public override void ResetValue(object component)
    {
        throw new InvalidOperationException("이 프로퍼티는 ReadOnly로만 동작되어야 한다.");
    }

    public override void SetValue(object component, object value)
    {
        throw new InvalidOperationException("이 프로퍼티는 ReadOnly로만 동작되어야 한다.");
    }

    public override bool ShouldSerializeValue(object component)
    {
        return false;
    }
    #endregion
}




public class DataForBinding : ICustomTypeDescriptor
{

    #region 사용하지 않는 ICustomTypeDescriptor methods
    public AttributeCollection GetAttributes()
    {
        return null;
    }

    public TypeConverter GetConverter()
    {
        return null;
    }

    public EventDescriptor GetDefaultEvent()
    {
        return null;
    }

    public PropertyDescriptor GetDefaultProperty()
    {
        return null;
    }

    public object GetEditor(Type editorBaseType)
    {
        return null;
    }

    public EventDescriptorCollection GetEvents(Attribute[] attributes)
    {
        return null;
    }

    public EventDescriptorCollection GetEvents()
    {
        return null;
    }

    public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
    {
        return null;
    } 
    #endregion

    IEnumerable<Data> originalDatas;
    string key;

    PropertyDescriptorCollection propertyDescriptorCollection = new PropertyDescriptorCollection(null, false);
    public DataForBinding(IEnumerable<Data> datas, IEnumerable<DateTime> datesForDisplay)
    {
        if (datas == null)
            throw new ArgumentNullException("datas");
        if (datas.Count() == 0)
            throw new ArgumentException();
        if (datesForDisplay == null)
        {
            throw new ArgumentException("datesForDisplay");
        }

        this.originalDatas = datas;
        this.key = datas.First().Type;
        if (!datas.All(d => d.Type == key))
        {
            //하나의 Row에 대응되는 Data들은 모두 Type이 동일해야 하므로
            throw new ArgumentException();
        }

        propertyDescriptorCollection.Add(new KeyPropertyDescriptor(key));
        foreach (DateTime dateTime in datesForDisplay)
        {
            propertyDescriptorCollection.Add(new DatePropertyDescriptor(datas, dateTime));
        }
    }

    public PropertyDescriptorCollection GetProperties()
    {
        return propertyDescriptorCollection;
    }

    public string GetClassName()
    {
        return typeof(DataForBinding).Name;
    }

    public string GetComponentName()
    {
        return typeof(DataForBinding).Name;
    }

    public object GetPropertyOwner(PropertyDescriptor pd)
    {
        return this;
    }
}

    public class Data
    {
        public string Type { get; set; }
        public DateTime DateTime { get; set; }
        public string Detail { get; set; }
    }

    public class DataCollection : List<Data> { }
}
